// Copyright 2023 Sirbu Dan
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include <io.hpp>

inline void setPinModeB(uint8_t pin, uint8_t mode)
{
    if (mode == INPUT) {
        DDRB &= ~(1 << pin);
        PORTB &= ~(1 << pin);
    } else if (mode == OUTPUT) {
        DDRB |= (1 << pin);
        PORTB &= ~(1 << pin);
    } else if (mode == INPUT_PULLUP) {
        DDRB &= ~(1 << pin);
        PORTB |= (1 << pin);
    }
}

inline void setPinModeC(uint8_t pin, uint8_t mode)
{
    if (mode == INPUT) {
        DDRC &= ~(1 << pin);
        PORTC &= ~(1 << pin);
    } else if (mode == OUTPUT) {
        DDRC |= (1 << pin);
        PORTC &= ~(1 << pin);
    } else if (mode == INPUT_PULLUP) {
        DDRC &= ~(1 << pin);
        PORTC |= (1 << pin);
    }
}

inline void setPinModeD(uint8_t pin, uint8_t mode)
{
    if (mode == INPUT) {
        DDRD &= ~(1 << pin);
        PORTD &= ~(1 << pin);
    } else if (mode == OUTPUT) {
        DDRD |= (1 << pin);
        PORTD &= ~(1 << pin);
    } else if (mode == INPUT_PULLUP) {
        DDRD &= ~(1 << pin);
        PORTD |= (1 << pin);
    }
}

inline uint8_t readB(uint8_t pin) {
    return PINB & (1 << pin);
}

inline uint8_t readC(uint8_t pin) {
    return PINC & (1 << pin);
}

inline uint8_t readD(uint8_t pin) {
    return PIND & (1 << pin);
}

inline void writeB(uint8_t pin, uint8_t value) {
    if (value == HIGH) {
        PORTB |= (1 << pin);
    } else if (value == LOW) {
        PORTB &= ~(1 << pin);
    }
}

inline void writeC(uint8_t pin, uint8_t value) {
    if (value == HIGH) {
        PORTC |= (1 << pin);
    } else if (value == LOW) {
        PORTC &= ~(1 << pin);
    }
}

inline void writeD(uint8_t pin, uint8_t value) {
    if (value == HIGH) {
        PORTD |= (1 << pin);
    } else if (value == LOW) {
        PORTD &= ~(1 << pin);
    }
}

void setPinMode(uint8_t pin, uint8_t mode)
{
    if (pin >= 0 && pin <= 7) {
        setPinModeD(pin, mode);
    } else if (pin > 7 && pin <= 13) {
        setPinModeB(pin - 8, mode);
    } else if (pin > 13 && pin <= 19) {
        setPinModeC(pin - 14, mode);
    }
}

uint8_t read(uint8_t pin)
{
    if (pin >= 0 && pin <= 7) {
        return readD(pin);
    } else if (pin > 7 && pin <= 13) {
        return readB(pin - 8);
    } else if (pin > 13 && pin <= 19) {
        return readC(pin - 14);
    }
}

void write(uint8_t pin, uint8_t value)
{
    if (pin >= 0 && pin <= 7) {
        writeD(pin, value);
    } else if (pin > 7 && pin <= 13) {
        writeB(pin - 8, value);
    } else if (pin > 13 && pin <= 19) {
        writeC(pin - 14, value);
    }
}